a:3:{i:0;a:3:{i:0;s:14:"document_start";i:1;a:0:{}i:2;i:0;}i:1;a:3:{i:0;s:4:"code";i:1;a:3:{i:0;s:10080:"
/***************************************************************************

  video.c

  Functions to emulate the video hardware of the machine.

***************************************************************************/

#include "emu.h"
#include "includes/1942.h"

/***************************************************************************

  Convert the color PROMs into a more useable format.

  1942 has three 256x4 palette PROMs (one per gun) and three 256x4 lookup
  table PROMs (one for characters, one for sprites, one for background tiles).
  The palette PROMs are connected to the RGB output this way:

  bit 3 -- 220 ohm resistor  -- RED/GREEN/BLUE
        -- 470 ohm resistor  -- RED/GREEN/BLUE
        -- 1  kohm resistor  -- RED/GREEN/BLUE
  bit 0 -- 2.2kohm resistor  -- RED/GREEN/BLUE

***************************************************************************/

PALETTE_INIT( 1942 )
{
	const UINT8 *color_prom = machine.root_device().memregion("proms")->base();
	rgb_t palette[256];
	int i, colorbase;

	for (i = 0; i < 256; i++)
	{
		int bit0, bit1, bit2, bit3, r, g, b;

		/* red component */
		bit0 = (color_prom[i + 0 * 256] >> 0) & 0x01;
		bit1 = (color_prom[i + 0 * 256] >> 1) & 0x01;
		bit2 = (color_prom[i + 0 * 256] >> 2) & 0x01;
		bit3 = (color_prom[i + 0 * 256] >> 3) & 0x01;
		r = 0x0e * bit0 + 0x1f * bit1 + 0x43 * bit2 + 0x8f * bit3;
		/* green component */
		bit0 = (color_prom[i + 1 * 256] >> 0) & 0x01;
		bit1 = (color_prom[i + 1 * 256] >> 1) & 0x01;
		bit2 = (color_prom[i + 1 * 256] >> 2) & 0x01;
		bit3 = (color_prom[i + 1 * 256] >> 3) & 0x01;
		g = 0x0e * bit0 + 0x1f * bit1 + 0x43 * bit2 + 0x8f * bit3;
		/* blue component */
		bit0 = (color_prom[i + 2 * 256] >> 0) & 0x01;
		bit1 = (color_prom[i + 2 * 256] >> 1) & 0x01;
		bit2 = (color_prom[i + 2 * 256] >> 2) & 0x01;
		bit3 = (color_prom[i + 2 * 256] >> 3) & 0x01;
		b = 0x0e * bit0 + 0x1f * bit1 + 0x43 * bit2 + 0x8f * bit3;

		palette[i] = MAKE_RGB(r,g,b);
	}

        // palette now contains 256 colors from color prom {8 bits red, 8 bits green, 8 bits blue} scaled by resistor values

	color_prom += 3 * 256;
	/* color_prom now points to the beginning of the lookup table */


	/* characters use palette entries 128-143 */

        // characters have 64 total colors available - built using character lookup table prom
        // entries in lookup table are 4 bits (256 of them)
        // use palette entries 128-143 (16 available unique colors)
        // occupy colorbase[255:0]

	colorbase = 0;
	for (i = 0; i < 64 * 4; i++)
		palette_set_color(machine, colorbase + i, palette[0x80 | *color_prom++]);
	colorbase += 64 * 4;

	/* background tiles use palette entries 0-63 in four banks */

        // bg tiles have 128 total colors available - built using bg lookup table prom
        // entries in lookup table are 4 bits (256 of them)
        // use palette entries 0-63 (64 available unique colors)
        // occupy colorbase[1023:256]
        // colorbase[511:256] are palette entries 0-15
        // colorbase[768:512] are palette entries 16-31
        // colorbase[1023:768] are palette entries 32-47
        // colorbase[1279:1024] are palette entries 48-63

	for (i = 0; i < 32 * 8; i++)
	{
		palette_set_color(machine, colorbase + 0 * 32 * 8 + i, palette[0x00 | *color_prom]);
		palette_set_color(machine, colorbase + 1 * 32 * 8 + i, palette[0x10 | *color_prom]);
		palette_set_color(machine, colorbase + 2 * 32 * 8 + i, palette[0x20 | *color_prom]);
		palette_set_color(machine, colorbase + 3 * 32 * 8 + i, palette[0x30 | *color_prom]);
		color_prom++;
	}
	colorbase += 4 * 32 * 8;

	/* sprites use palette entries 64-79 */

        // sprites have 16 total colors available - built using sprite lookup table prom
        // entries in lookup table are 4 bits (256 of them)
        // use palette entries 64-79 (16 available unique colors)
        // occupy colorbase[1535:1280]

	for (i = 0; i < 16 * 16; i++)
		palette_set_color(machine, colorbase + i, palette[0x40 | *color_prom++]);
}


/***************************************************************************

  Callbacks for the TileMap code

***************************************************************************/

static TILE_GET_INFO( get_fg_tile_info )
{
	_1942_state *state = machine.driver_data<_1942_state>();
	int code, color;

	code = state->m_fg_videoram[tile_index];
	color = state->m_fg_videoram[tile_index + 0x400];
	SET_TILE_INFO(
			0, 
			code + ((color & 0x80) << 1),
			color & 0x3f,
			0);

        // color is index + 1024 (this is in color ram d400-d7ff)
        // use gfx[0]
        // index of graphics for this tile = code + (8th bit of color left shifted one)
        // color to use for this tile ignores 8th and 7th bit (64 colors available)

}

static TILE_GET_INFO( get_bg_tile_info )
{
	_1942_state *state = machine.driver_data<_1942_state>();
	int code, color;

	tile_index = (tile_index & 0x0f) | ((tile_index & 0x01f0) << 1);

        // tileindex = {tile_index[8:4], 0, tile_index[3:0]}

	code = state->m_bg_videoram[tile_index];
	color = state->m_bg_videoram[tile_index + 0x10];
	SET_TILE_INFO(
			1,
			code + ((color & 0x80) << 1),
			(color & 0x1f) + (0x20 * state->m_palette_bank),
			TILE_FLIPYX((color & 0x60) >> 5));

        // color is index + 16 (bytes) (16 bytes code, 16 bytes color/attribute)
        // use gfx[1]
        // index of graphics for this tile = code + (8th bit of color left shifted one)
        // color to use for this tile is {0x20*m_palette_bank, color[4:0]} (128 colors available - 4banks*2^5)
        // flipy if color[6], flipx if color[7]
        
}


/***************************************************************************

  Start the video hardware emulation.

***************************************************************************/
VIDEO_START( 1942 )
{
	_1942_state *state = machine.driver_data<_1942_state>();
	state->m_fg_tilemap = tilemap_create(machine, get_fg_tile_info, tilemap_scan_rows, 8, 8, 32, 32);

        // fg tilemap is 256x256 - scanned in row major order
        // 8x8 tiles

	state->m_bg_tilemap = tilemap_create(machine, get_bg_tile_info, tilemap_scan_cols, 16, 16, 32, 16);

        // bg tilemap is 512x256 - scanned in column major order
        // 16x16 tiles

	state->m_fg_tilemap->set_transparent_pen(0);
}


/***************************************************************************

  Memory handlers

***************************************************************************/

WRITE8_MEMBER(_1942_state::c1942_fgvideoram_w)
{

	m_fg_videoram[offset] = data;
	m_fg_tilemap->mark_tile_dirty(offset & 0x3ff);
}

WRITE8_MEMBER(_1942_state::c1942_bgvideoram_w)
{

	m_bg_videoram[offset] = data;
	m_bg_tilemap->mark_tile_dirty((offset & 0x0f) | ((offset >> 1) & 0x01f0));
}


WRITE8_MEMBER(_1942_state::c1942_palette_bank_w)
{

	if (m_palette_bank != data)
	{
		m_palette_bank = data;
		m_bg_tilemap->mark_all_dirty();
	}
}

WRITE8_MEMBER(_1942_state::c1942_scroll_w)
{

	m_scroll[offset] = data;
	m_bg_tilemap->set_scrollx(0, m_scroll[0] | (m_scroll[1] << 8));

        // scroll bg tilemap (0 indicates that whole tilemap scrolls as one)
        // scroll by {m_scroll[1], m_scroll[0]} pixels
        // offset is either 0 or 1
}


WRITE8_MEMBER(_1942_state::c1942_c804_w)
{
	/* bit 7: flip screen
       bit 4: cpu B reset
       bit 0: coin counter */

	coin_counter_w(machine(), 0,data & 0x01);

	device_set_input_line(m_audiocpu, INPUT_LINE_RESET, (data & 0x10) ? ASSERT_LINE : CLEAR_LINE);

	flip_screen_set(data & 0x80);
}


/***************************************************************************

  Display refresh

***************************************************************************/

static void draw_sprites( running_machine &machine, bitmap_ind16 &bitmap, const rectangle &cliprect )
{
	_1942_state *state = machine.driver_data<_1942_state>();
	int offs;

        // loop through sprite rom, highest addresses first (4 bytes at a time)

	for (offs = state->m_spriteram.bytes() - 4; offs >= 0; offs -= 4)
	{
		int i, code, col, sx, sy, dir;

		code = (state->m_spriteram[offs] & 0x7f) + 4 * (state->m_spriteram[offs + 1] & 0x20)
				+ 2 * (state->m_spriteram[offs] & 0x80);

                // code (offset to graphics element) = {m_spriteram[offs][7], 0, m_spriteram[offs + 1][5], m_spriteram[offs][6:0]}


		col = state->m_spriteram[offs + 1] & 0x0f;
                
                // col = m_spriteram[offs + 1][3:0]
                // col is color

		sx = state->m_spriteram[offs + 3] - 0x10 * (state->m_spriteram[offs + 1] & 0x10);

		sy = state->m_spriteram[offs + 2];

		dir = 1;

		if (state->flip_screen())
		{
			sx = 240 - sx;
			sy = 240 - sy;
			dir = -1;
		}

		/* handle double / quadruple height */
		i = (state->m_spriteram[offs + 1] & 0xc0) >> 6;

                // 0 <= i <= 3

		if (i == 2)
			i = 3;

                // i is either 0,1,3

		do
		{
			drawgfx_transpen(bitmap,
                                         cliprect,
                                         machine.gfx[2],
					 code + i,
                                         col,
					 state->flip_screen(),
                                         state->flip_screen(),
					 sx,
                                         sy + 16 * i * dir,
                                         15);

                        // void drawgfx_transpen(
                        // bitmap_ind16 &dest,
                        // const rectangle &cliprect,
                        // gfx_element *gfx,
                        // UINT32 code,
                        // UINT32 color,
                        // int flipx,
                        // int flipy,
                        // INT32 destx,
                        // INT32 desty,
                        // UINT32 transpen);

			i--;
		} while (i >= 0);
	}
}

SCREEN_UPDATE_IND16( 1942 )
{
	_1942_state *state = screen.machine().driver_data<_1942_state>();

	state->m_bg_tilemap->draw(bitmap, cliprect, 0, 0);
	draw_sprites(screen.machine(), bitmap, cliprect);
	state->m_fg_tilemap->draw(bitmap, cliprect, 0, 0);
	return 0;
}
";i:1;N;i:2;N;}i:2;i:6;}i:2;a:3:{i:0;s:12:"document_end";i:1;a:0:{}i:2;i:6;}}